package com.hz.pm.api.projectlib.handle;

import com.google.common.collect.Maps;
import com.wflow.workflow.bean.vo.ProcessDetailVO;
import com.wflow.workflow.bean.vo.ProcessInfoVO;
import com.wflow.workflow.enums.ProcessStatusEnum;
import com.wflow.workflow.enums.StepStatusEnum;
import lombok.AllArgsConstructor;
import lombok.Getter;
import org.springframework.core.Ordered;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 流程业务抽象处理类
 *
 * @author CMM
 * @since 2023/02/24 14:22
 */

public abstract class AbstractProcessHandle implements Ordered {
    /**
     * 构造，子类必须实现
     */
    protected AbstractProcessHandle() {

    }

    /**
     * 抽象的，所有具体处理者应该实现的处理逻辑
     *
     * @param ctx ctx
     */
    abstract void handle(HandleParamContext ctx);

    abstract ProcessNode getProcessNode();

    @Getter
    @AllArgsConstructor
    public enum ProcessNode {

        PROJECT_DECLARED("项目申报", 1),
        PROJECT_REVIEW("项目评审", 2),
        APPROVAL_AMOUNT("资金下达", 3),
        PROJECT_APPROVAL("项目立项", 4),
        PROJECT_PURCHASE("招标采购", 5),
        PROJECT_ADAPTION("适配改造", 6),
        SYSTEM_SELF_TEST("系统自测", 7),
        SYSTEM_TEST_VALID("测试验证", 8),
        FIRST_INSPECTED("项目初验", 9),
        PILOT_RUNNING("上线试运行", 10),
        XCFHX_REVIEW("信创测评", 11),
        FINAL_INSPECTED("项目终验", 12);


        private final String name;
        private final Integer order;

        public static final List<ProcessNode> ALL = Arrays.stream(ProcessNode.values())
                .sorted(Comparator.comparingInt(ProcessNode::getOrder))
                .collect(Collectors.toList());

    }

    private static final Map<ProcessStatusEnum, Integer> PROCESS_STATUS_PRIORITY_MAP = new HashMap<>();

    static {
        PROCESS_STATUS_PRIORITY_MAP.put(ProcessStatusEnum.APPROVED, 3);
        PROCESS_STATUS_PRIORITY_MAP.put(ProcessStatusEnum.UNDER_REVIEW, 1);
        PROCESS_STATUS_PRIORITY_MAP.put(ProcessStatusEnum.BE_REJECTED, 2);
    }

    static StepStatusEnum getPurchaseStatus(List<ProcessInfoVO> processes) {
        Optional<Integer> min = processes.stream().map(w -> {
            ProcessStatusEnum status = Arrays.stream(ProcessStatusEnum.values())
                    .filter(s -> s.getDesc().equals(w.getProcessProgressVo().getStatus()))
                    .findFirst()
                    .orElse(ProcessStatusEnum.NOT_SUBMITTED);
            return PROCESS_STATUS_PRIORITY_MAP.getOrDefault(status, 0);
        }).min(Comparator.comparingInt(Integer::intValue));
        if (min.isPresent()) {
            int miniOrder = min.get();
            if (miniOrder == 1) {
                return StepStatusEnum.ON_GOING;
            } else if (miniOrder == 2) {
                return StepStatusEnum.REJECTED;
            } else {
                return StepStatusEnum.COMPLETED;
            }
        } else {
            return StepStatusEnum.NOT_START;
        }
    }

}
